'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Node = exports.Connections = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _immutable = require('immutable');

var _state = require('./state.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Connections = exports.Connections = function () {
  function Connections(nodes, badNodes) {
    _classCallCheck(this, Connections);

    this.knownNodes = (0, _immutable.List)();
    this.knownBadNodes = (0, _immutable.List)();
    if (nodes != undefined) this.knownNodes = (0, _immutable.List)(nodes);
    if (badNodes != undefined) this.knownBadNodes = (0, _immutable.List)(badNodes);
  }

  _createClass(Connections, [{
    key: 'randomNode',
    value: function randomNode() {
      return this.knownNodes.get(Math.floor(Math.random() * this.knownNodes.size));
    }
  }, {
    key: 'addNode',
    value: function addNode(host, port) {
      // If host is undefined, we need to see if it's a Node and add it if so.
      if (port != undefined) {
        this.knownNodes = this.knownNodes.push(new Node(host, port));
        return true;
      } else {
        if (host.host !== undefined && host.port !== undefined) {
          this.knownNodes = this.knownNodes.push(host);
          return true;
        } else {
          return false;
        }
      }
    }
  }, {
    key: 'pruneNodes',
    value: function pruneNodes() {
      var existing_nodes = [];
      var newNodes = (0, _immutable.List)();
      this.knownNodes.map(function (node) {
        var node_string = node.host + ':' + node.port.toString();
        if (_.contains(existing_nodes, node_string)) return;
        existing_nodes.push(node_string);
        newNodes = newNodes.push(node);
      });
      this.knownNodes = newNodes;
    }
  }, {
    key: 'toJson',
    value: function toJson() {
      var response = {
        knownNodes: [],
        knownBadNodes: []
      };
      this.knownNodes.map(function (node) {
        response["knownNodes"].push(node.toJson());
      });
      this.knownBadNodes.map(function (node) {
        response["knownBadNodes"].push(node.toJson());
      });
      return response;
    }
    /*
      Be aware this appends Nodes being imported to ones you already have in your Connections object.
      Utilize the `prune_nodes` method to remove duplications.
    */

  }, {
    key: 'importJson',
    value: function importJson(payload) {
      if (typeof payload === 'string') {
        payload = JSON.parse(payload);
      }
      for (var key in payload.knownNodes) {
        var n = payload.knownNodes[key];
        var node = new Node(n.host, n.port, n.authToken, n.apiVersion);
        this.knownNodes = this.knownNodes.push(node);
      }
      for (var key in payload.knownBadNodes) {
        var n = payload.knownBadNodes[key];
        var node = new Node(n.host, n.port, n.authToken, n.apiVersion);
        this.knownBadNodes = this.knownBadNodes.push(node);
      }
    }
  }]);

  return Connections;
}();

var Node = exports.Node = function () {
  function Node(host, port, authToken, apiVersion) {
    _classCallCheck(this, Node);

    this.host = host;
    this.port = Number(port);

    this.authToken = null;
    this.apiVersion = 'v1';
    if (authToken !== undefined) this.authToken = authToken;
    if (apiVersion !== undefined) this.apiVersion = apiVersion;
  }

  _createClass(Node, [{
    key: 'toJson',
    value: function toJson() {
      return {
        host: this.host,
        port: this.port,
        authToken: this.authToken,
        apiVersion: this.apiVersion
      };
    }
  }, {
    key: 'get',
    value: function get(endpoint, qs, cb) {
      if (qs === undefined) {
        qs = {};
      }

      return (0, _request2.default)({
        url: 'http://' + this.host + ':' + this.port + '/api/' + this.apiVersion + '/' + endpoint,
        qs: qs,
        method: 'GET'
      }, function (error, response, body) {
        cb(body);
      });
    }
  }, {
    key: 'post',

    // Pass formData in as a JSON object of key value pairs.
    value: function post(endpoint, formData, cb) {
      return (0, _request2.default)({
        uri: 'http://' + this.host + ':' + this.port + '/api/' + this.apiVersion + '/' + endpoint,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        json: formData
      }, function (error, response, body) {
        cb(body);
      });
    }
  }, {
    key: 'status',


    // Get status of a particular node.
    value: function status(cb) {
      return this.get('status', {}, cb);
    }
  }, {
    key: 'authenticate',


    // Authenticates a node with a credential you pass in.
    value: function authenticate(credential, cb) {
      var node = this;
      this.get('auth', {}, function (response) {
        var signature = credential.signHex(JSON.parse(response).challenge)[0];
        var formData = {
          'challenge': signature
        };
        node.post('auth', formData, function (response) {
          var authToken = response.authToken;
          node.authToken = authToken;
          cb(authToken);
        });
      });
    }
  }, {
    key: 'getState',
    value: function getState(cb) {
      return this.get('state', {}, cb);
    }
  }, {
    key: 'searchState',
    value: function searchState(fqdn, type, address, cb) {
      return this.getState(function (response) {
        cb((0, _state.searchState)(response, fqdn, type, address));
      });
    }
  }, {
    key: 'searchTransactions',
    value: function searchTransactions(query_params, cb) {
      return this.get('transaction', query_params, cb);
    }
  }, {
    key: 'getTransaction',
    value: function getTransaction(txhash, cb) {
      return this.get('transaction/' + txhash, {}, cb);
    }
  }, {
    key: 'submitTransaction',
    value: function submitTransaction(tx_params, cb) {
      return this.post('transaction', tx_params, cb);
    }
  }]);

  return Node;
}();